;; Copyright 2009, 2010, 2011, 2013, 2014, 2015, 2016 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.




;;-----------------------------------------------------------------------------
;; filladapt-pat-gp-comment   \\

(filladapt-debug)
(gp-filladapt-setup)
filladapt-token-table
filladapt-token-match-table
filladapt-token-conversion-table
(setq comment-start-skip "\\\\\\\\.*$\\|/\\*\\([^*]\\|\\*+[^*/]\\)*\\*+/")
(setq comment-start-skip "\\\\\\\\+[ \t]*\\|/\\*+[ \t]*")

\\ dsfjks
\\ dsfjks
\\ dsfjks
\\ dsfjks 
\\ dsfjks 


;; When filladapt.el is loaded, add a `filladapt-pat-gp-comment' entry for \\ style GP
;; comments.
;;
;; If filladapt.el is not yet loaded then an `eval-after-load' applies the
;; setup when filladapt.el later loads.

(require 'derived) ;; for emacs21 since `derived-mode-p' is not preloaded

;; this would be in the `gp-script-mode' code
(add-hook 'gp-script-mode-hook 'gp-filladapt-setup)

  (when (boundp 'filladapt-token-table)
    (make-local-variable 'filladapt-token-table)
    ;; filladapt.el says `filladapt-token-table' must have its
    ;; `beginning-of-line' as its first entry, so append here
    (add-to-list 'filladapt-token-table
                 '("\\\\\\\\+" filladapt-pat-gp-comment)
                 t))) ;; append

(defun gp-filladapt-after-load ()
  "Apply PARI/GP setups to filladapt.el.
A `filladapt-pat-gp-comment' type \\=\\\\=\\ is created and if there are any
`gp-script-mode' buffers then `filladapt-pat-gp-comment' is added buffer-local
to `filladapt-token-table' in each.  If or when `filladapt-mode'
is used in those buffers it will have \\=\\\\=\\ as a fill prefix.

`filladapt-token-table' is only changed buffer-local since done
globally it could make a mess of TeX \\=\\\\=\\ row separator macro.

If \\=\\\\=\\ was a global fill prefix then such a TeX macro at
the start of the line could cause another \\=\\\\=\\ to be added
on the next line, making a mess of rows."

  (add-to-list 'filladapt-token-match-table      '(filladapt-pat-gp-comment filladapt-pat-gp-comment))
  (add-to-list 'filladapt-token-conversion-table '(filladapt-pat-gp-comment . exact))
  (mapcar (lambda (buffer)
            (with-current-buffer buffer
              (when (derived-mode-p 'gp-script-mode)
                (gp-filladapt-setup))))
          (buffer-list)))

(eval-after-load "filladapt" '(gp-filladapt-after-load))


;;-----------------------------------------------------------------------------
;; dnl

(filladapt-debug)
(filladapt-pat-dnl)

dnl dsfjks
dnl dsfjks 
dnl dsfjks
dnl dsfjks 
dnl dsfjks 

;;-----------------------------------------------------------------------------
;; LocalWords

(featurep 'filladapt-pat)
(require 'filladapt-pat)
(filladapt-pat-after-load)
filladapt-token-table
(load (expand-file-name "../filladapt-pat.el"))
(filladapt-pat-LocalWords t)
(setq filladapt-pat-globally '(filladapt-pat-LocalWords))
(custom-set-variables '(filladapt-pat-globally (filladapt-pat-LocalWords)))
;; LocalWords: aaa
;; LocalWords: bbb
;; LocalWords: ccc fjdks fjks dfjks 
;; LocalWords: aaa
;; LocalWords: aaa

;;-----------------------------------------------------------------------------
;; globally

(setq filladapt-pat-globally '(filladapt-pat-no-postscript))
(filladapt-pat-after-load)

(filladapt-debug)
(filladapt-pat-no-postscript)

# comment about
# %FOO blah blah
# blah blah


;;-----------------------------------------------------------------------------
;; elisp docstring first line

(flyspell-mode 0)
(filladapt-debug)
(my-filladapt-pat-docstring-first-bullet)
(my-parasep-docstring-first-line)
(text-mode)
filladapt-token-table
paragraph-start

(defun foo ()
  "First line   of   docstring.
Second linef dsjf sjf sjdkf jsdk fjksd fjksd jfk sdjfk sdkjf sdk fjksd fjksd jfk sdjfk sdkjf sdkf jsdk fk.
Third linef dsjf sjf sjdkf jsdk fjksd fjksd jfk sdjfk sdkjf sdk fjksd fjksd jfk sdjfk sdkjf sdkf jsdk fk."
  nil)


;;-----------------------------------------------------------------------------
;; perl vs postscript

(filladapt-debug)
(filladapt-pat-no-postscript)

# comment about
# %FOO blah blah
# blah blah

;;-----------------------------------------------------------------------------
;; pod vs supercite

(filladapt-debug)
(filladapt-pat-no-supercite)

some pod C<code
break> blah blah
blah blah blah

;;-----------------------------------------------------------------------------
;; filladapt-pat-bullet-pod

(filladapt-debug)
(filladapt-pat-bullet-pod)

=item some thing fjkds fjkds fjksd fjks djf sjdkf jksd fkjsd fkj sdjkf sk jfksd jfkds fjksd
xxxx xxxxxxx xxxxxxxx xxxxxxxx

=item some thing fjkds fjkds fjksd fjks djf sjdkf jksd fkjsd fkj sdjkf
      sk jfksd jfkds fjksd


;;-----------------------------------------------------------------------------
;; filladapt-pat-version-bullet

(filladapt-debug)
(filladapt-pat-version-bullet)
(parasep-add "Version [0-9]+ +- ")
(setq paragraph-start (concat paragraph-start "\\|Version [0-9]+ +- "))
filladapt-token-table

Version 1 - some thing fjkds fjkds fjksd fjks djf sjdkf jksd fkjsd fkj sdjkf sk jfksd jfkds fjksd

- some thing fjkds fjkds fjksd fjks djf sjdkf jksd fkjsd fkj sdjkf sk jfksd jfkds fjksd

Version 1 - some thing fjkds fjkds fjksd fjks djf sjdkf jksd fkjsd fkj sdjkf sk jfksd jfkds fjksd
Version 2 - some thing fjkds fjkds fjksd fjks djf sjdkf jksd fkjsd fkj sdjkf
      sk jfksd jfkds fjksd

;; Version 1 - some thing fjkds fjkds fjksd fjks djf sjdkf jksd fkjsd fkj fjkdsl jfsdkl fskld 
;;             sdjkf sk jfksd jfkds fjksd
;; Version 2 - some thing fjkds fjkds fjksd fjks djf sjdkf jksd fkjsd fkj sdjkf
      sk jfksd jfkds fjksd

;; Version 1.2 - fdjksfjkds jfks jfks jfk sdjfk sjkf fjdks
;;               sdkf jsdk fjskd fjks fjksd fjksd fjkds fjks fjkds fjsdk


;;-----------------------------------------------------------------------------

(progn
  (require 'cl)
  (defun foo (x)
    (remove* x token-table :key 'cadr))
  (byte-compile 'foo)
  (disassemble 'foo))
