;; Copyright 2009, 2010, 2015 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


(add-to-list 'Info-default-directory-list "/down/emacs/info")
(info-lookup-reset)
(load (expand-file-name "info-lookmore.el"))
(info-lookmore-elisp-userlast)
(info-lookup-symbol "beginning-of-buffer")



;;-----------------
;; byte-compile

(progn
  (add-to-list 'load-path (expand-file-name "."))
  (require 'my-byte-compile)
  (my-byte-compile))

;;-----------------------------------------------------------------------------
;; coreutils

(progn
  (setq info-lookup-alist nil)
  (info-lookup-reset)
  (info-lookup-maybe-add-help
   :mode 'sh-mode
   :topic 'symbol
   :regexp "\\([a-zA-Z0-9_-]+\\|[!{}@*#?$]\\|\\[\\[?\\|]]?\\)"
   :doc-spec '(
               ;; coreutils (version 4.5.10) doesn't have a separate program
               ;; index, so exclude extraneous stuff (most of it) by demanding
               ;; "[a-z]+" in the trans-func.
               ("(coreutils)Index"
                (lambda (item)
                  (debug)
                  (if (string-match "\\`[a-z]+\\'" item)
                      item)))
               ))
  (info-lookup-symbol "cp" 'sh-mode))
cp

(Info-goto-node "(coreutils)Concept Index")

(progn
  (setq info-lookup-alist nil)
  (info-lookup-reset))

(progn
  (switch-to-buffer "x")
  (pp info-lookup-cache (current-buffer)))


;;-----------------------------------------------------------------------------
(info-lookmore-c-readline)
(info-lookup-symbol "readline" 'c-mode)
(info-lookup-symbol "add_history" 'c-mode)

;;-----------------------------------------------------------------------------
(info-lookmore-c++-use-c)
(info-lookup-symbol "printf" 'c++-mode)
(info-lookup-file "math.h" 'c++-mode)
(info-lookup-file "stdio.h" 'c++-mode)
(info-lookup-file "stdlib.h" 'c++-mode)
(info-lookup-file "unistd.h" 'c++-mode)
(info-lookup-file "wchar.h" 'c++-mode)


;;-----------------------------------------------------------------------------
(info-lookmore-c-gcc)
(info-lookup-symbol "__builtin_expect" 'c-mode)
(info-lookup-symbol "__complex__" 'c-mode)
(info-lookup-symbol "__float128" 'c-mode)
(info-lookup-symbol "__func__" 'c-mode)
(info-lookup-symbol "__FUNCTION__" 'c-mode)
(info-lookup-symbol "__PRETTY_FUNCTION__" 'c-mode)
(info-lookup-symbol "empty" 'c-mode)
(info-lookup-symbol "empty declaration" 'c-mode)
(info-lookup-reset)

(info-lookmore-modify-doclist 'symbol 'c-mode (lambda (doc-list) nil))

;;-----------------------------------------------------------------------------
(progn
  (load (expand-file-name "../info-lookmore.el"))
  (info-lookmore-elisp-gnus)
  )
(info-lookup-symbol "gnus" 'emacs-lisp-mode)
(info-lookup-symbol "nnrss-opml-import" 'emacs-lisp-mode)     ;; in gnus
(info-lookup-symbol "nnrss-directory" 'emacs-lisp-mode)       ;; in gnus
(info-lookup-symbol "mm-file-name-rewrite-functions" 'emacs-lisp-mode) ;; in gnus
(info-lookup-symbol "ietf-drums-parse-date" 'emacs-lisp-mode) ;; in emacs-mime
(info-lookup-symbol "yenc" 'emacs-lisp-mode)                  ;; in emacs-mime
(info-lookup-symbol "Chinese" 'emacs-lisp-mode)
(info-lookup-symbol "interface" 'emacs-lisp-mode)

;; not found by prefix/suffix patterns as not at start of line
(info-lookup-symbol "mm-file-name-trim-whitespace" 'emacs-lisp-mode)

(info-lookup-reset)
info-lookup-cache
info-lookup-alist

(info-lookmore-modify-doclist 'symbol 'emacs-lisp-mode (lambda (doc-list) nil))

(re-search-forward (concat "^\\( --? .*: \\|[`'‘]\\)" "ietf-drums-parse-date" "\\([ '’]\\|$\\)"))
(re-search-forward "^\\( --? .*: \\|[`']\\)ietf-drums-parse-date\\([ ']\\|$\\)")

;;-----------------------------------------------------------------------------

(progn
  (defun my-foo ()
    (concat "x" "y" zz "ab" "cd"))
  (byte-compile 'my-foo)
  (disassemble 'my-foo))

(byte-optimize-concat '(concat x y))
(byte-optimize-concat '(concat "x"))
(byte-optimize-concat '(concat "x" "y"))
(byte-optimize-concat '(concat "x" "y" z))


(progn
  (defun foo ()
    (bar
     'symbol 'sh-mode
     (eval-when-compile
       `("(coreutils)Concept Index"
         ,(lambda (item) (if (string-match "\\`[a-z]+\\'" item) item))))))
  (byte-compile 'foo)
  (disassemble 'foo))



;;-----------------------------------------------------------------------------
eg. describe-character-set

\` const if all \, are const ...

(defun my-foo ()
  `(,1 2 3))
(defun my-foo ()
  `(1 ,2))

(my-foo)
(pp-to-string (symbol-function 'my-foo))
(let ((print-quoted nil))
  (prin1 (symbol-function 'my-foo)))

(defun my-foo ()
  (progn ;;eval-when-compile
    `(1 2 ,(+ 3 4) 5 6 7)))
(defun my-foo ()
  (progn ;; eval-when-compile
    `(1 2 ,(lambda () (+ 3 4)) 5)))
(byte-compile 'my-foo)
(disassemble 'my-foo)

(defalias 'backquote-list* 'backquote-list*-function)
(defalias 'backquote-list* 'backquote-list*-macro)
(debug-on-entry 'byte-optimize-form)
(cancel-debug-on-entry 'byte-optimize-form)

(backquote-process '(1 (\, (+ 2 3)) 4 5 6))
(backquote-process '(1 (\, (lambda () 999)) 4 5 6))
`(1 ,(+ 2 3) 4 5 6)
(debug-on-entry 'backquote-process)
(cancel-debug-on-entry 'backquote-process)

(byte-compile-constp (lambda () 999))
(byte-compile-constp 999)

(put 'backquote-list* 'pure t)
(put 'backquote-list*-function 'pure t)
(put 'backquote-list*-macro 'pure t)



;;-----------------------------------------------------------------------------

(defun info-lookmore-file-exists-p ()
  (require 'info)
  (condition-case err
      (Info-find-file "termcap")
    (error
     (if (save-match-data
           (string-match "Info file .* does not exist"
                         (error-message-string err)))
         nil
       (signal (car err) (cdr err))))))


;;-----------------------------------------------------------------------------
;; a sit-for delay way back on an unknown node is gone, so no great harm to
;; leave termcap in without the info file

(defun info-lookmore-file-exists-p (filename)
  ;; This is fairly diabolical, but alas
  ;;    * `Info-find-file' is only in recent emacs (and it throws an error
  ;;      for not found, which is a bit unfriendly).
  ;;    * `info-xref-goto-node-p' is a bit slack not preserving the info
  ;;      history, and might be a bit slow fontifying etc
  ;;    * `Info-suffixed-file' is only in xemacs
  (require 'info)
  (require 'cl)
  (info-initialize)
  (some (lambda (dir)
          (let ((filename (expand-file-name filename dir)))
            (some (lambda (suffix)
                    (file-exists-p (concat filename suffix)))
                  (cons "" (mapcar 'car Info-suffix-list)))))
        (append Info-directory-list
                (and (boundp 'Info-additional-directory-list) ;; emacs
                     Info-additional-directory-list)
                (and (boundp 'Info-additional-search-directory-list) ;; xemacs
                     Info-additional-search-directory-list))))

;;;###autoload
(defun info-lookmore-check-termcap ()
  "Remove the termcap manual from `c-mode'."
  (unless (info-lookmore-file-exists-p "termcap")
    (info-lookmore-remove-doc 'symbol 'c-mode "(termcap)Var Index")

(autoload 'info-lookmore-check-termcap "info-lookmore")
(eval-after-load "info-look" '(info-lookmore-check-termcap))


;;-----------------------------------------------------------------------------
;; grep for info index entries starting with a "*", such as
;; bad pointer return value forms like "*__builtin_assume_aligned"
(grep (concat "zgrep -nH '^[*] [*]' "
              " /usr/share/info/*.info*.gz"
              " /usr/share/info/*/*.info*.gz"
              " /usr/share/info/*.info*"
              " /usr/share/info/*/*.info*"))
