;; Copyright 2003, 2004, 2007, 2009, 2010 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


;; Look for interactive commands, ie. anything `commandp', which don't have
;; docstrings.



'(progn
  (load (expand-file-name "my-load-all.el"))
  (my-load-all--autoloads))

(defun my-undocumented-commands ()
  "Return a list of symbols which are commands without docstrings."
  (let ((bad nil))
    (mapatoms (lambda (sym)
                (if (and (commandp sym)
                         (not (my-command-autoloadp sym))
                         (not (documentation sym t)))
                    (setq bad (cons sym bad)))))
    bad))

(defun my-symbol-file+name-lessp (x y)
  "Return t if X < Y for two symbols, compared by `symbol-file' then by name.
If `symbol-file' is nil, it's considered less than any proper filename."
  (let ((xf (or (symbol-file x) ""))
        (yf (or (symbol-file y) "")))
    (if (string-equal xf yf)
        (string-lessp (symbol-name x) (symbol-name y))
      (string-lessp xf yf))))

(defun my-symbol-file (symbol)
  "Like `symbol-file', but ensure a \".el\", and give \"unknown file\"."
  (let ((name (symbol-file symbol)))
    (if name        
        (if (string-match "\\.el\\'" name)
            name
          (concat name ".el"))
      "unknown file")))

(defun my-command-autoloadp (sym)
  "Return `t' if SYM is an `autoload' function."
  (let ((func (symbol-function sym)))
    (and (consp func)
         (eq 'autoload (car func)))))

(defun my-autoload-file (command)
  "Get the file from an `autoload' COMMAND."
  (cadr (symbol-function command)))



(progn
  (switch-to-buffer "*emacs-undocumented-commands*")
  (erase-buffer)
  (dolist (i (sort (my-undocumented-commands) 'my-symbol-file+name-lessp))
    (insert (format "%s: undocumented command: %s%s\n"
                    (my-symbol-file i)
                    (symbol-name i)
                    (if (my-command-autoloadp i)
                        (format "  [autoload \"%s\"]" (my-autoload-file i))
                      "")))))
