;; Copyright 2009, 2010, 2011, 2012 Kevin Ryde

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defconst my-run-test--not-emacs20-p
  (not (string-match "\\`20\\." emacs-version)))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  (let ((count 0))

    ;; check the autoload cookie eval-after-load
    (require 'compile)
    (or (featurep 'compilation-perl)
	(error "compilation-perl.el not loaded with compile.el"))
    (setq count (1+ count))


    (add-to-list 'load-path (expand-file-name "."))
    (require 'my-compile-testing)

    (my-compilation-validate-alists)

    ;; each WANT element per `my-compilation-error-lines' in
    ;; my-compile-testing.el, being
    ;;     (LINENUM TARGET-FILENAME TARGET-LINENUM)
    ;;
    (mapcar (lambda (elem)
              (let ((filename (nth 0 elem))
                    (want     (nth 1 elem))
                    (condsym  (nth 2 elem)))
                (if (and condsym
                         (not (symbol-value condsym)))
                    (message "skip %S due to %S" filename condsym)

                  (find-file filename)
                  (compilation-mode)
                  (let ((got (my-compilation-error-lines)))

                    ;; emacs 24.1 patterns too loose, they match "Current time" bits
                    (setq got (remove '(30 "Current time local: Thu Feb 18 16:49" 46) got))
                    (setq got (remove '(31 "Current time GMT:   Thu Feb 18 05:49" 46) got))
                    (setq got (remove '(30 "Current time local: Sat Dec 29 11:17" 19) got))
                    (setq got (remove '(31 "Current time GMT:   Sat Dec 29 00:17" 19) got))
                    (setq got (remove '(30 "Current time GMT:   Thu Feb 18 05:29" 48) got))
                    (setq got (remove '(29 "Current time local: Thu Feb 18 16:29" 48) got))

                    ;; compile.el perl regexp breakage in emacs 23.1
                    ;; FIXME: might try to fix it in compilation-perl.el
                    '(when (and (equal filename "t-compilation-perl--Glib.txt")
                                (equal emacs-major-version 23)
                                (equal emacs-minor-version 1))
                       (setq got want))

                    (unless (equal got want)
                      (require 'pp) ;; not pre-loaded in emacs21
                      '(message "compilation-error-regexp-alist %S"
                                compilation-error-regexp-alist)
                      (error "wrong error lines for %S\ngot\n%s\nwant\n%s"
                             filename
                             (pp-to-string got)
                             (pp-to-string want))))
                  (kill-buffer nil)
                  (setq count (1+ count)))))

            '(
              ("t-compilation-perl--Test-raw-5.8.4.txt"
               ((34 "../devel/d-compilation-perl.t" 27)
                (37 "../devel/d-compilation-perl.t" 28)
                (41 "../devel/d-compilation-perl.t" 29)))
              ("t-compilation-perl--Test-raw-5.8.8.txt"
               ((34 "t-compilation-perl.t" 5)
                (37 "t-compilation-perl.t" 6)
                (40 "t-compilation-perl-2.t" 8)
                (42 "t-compilation-perl-2.t" 10)))
              ("t-compilation-perl--Test-raw-5.10.1.txt"
               ((33 "d-compilation-perl.t" 27)
                (36 "d-compilation-perl.t" 28)
                (40 "d-compilation-perl.t" 29)
                (44 "devel/d-compilation-perl-2.t" 33)))

              ("t-compilation-perl--Test::Harness-5.8.8.txt"
               ((28 "d-compilation-perl.t" 27)
                (30 "d-compilation-perl.t" 28)
                (33 "d-compilation-perl.t" 29)))
              ("t-compilation-perl--Test::Harness-5.10.1.txt"
               ((30 "../devel/d-compilation-perl.t" 27)
                (32 "../devel/d-compilation-perl.t" 28)
                (35 "../devel/d-compilation-perl.t" 29)))

              ("t-compilation-perl--global-destruction.txt"
               ((26 "foo.pl" 3)))

              ("t-compilation-perl--Glib.txt"
               ((31 "t-compilation-perl-gtk.pl" 3)))

              ("t-compilation-perl--xsubpp.txt"
               ((26 "t-compilation-perl.xs" 11)
                (27 "t-compilation-perl.xs" 38)
                (28 "d-compilation-perl.xs" 44)))

              ("t-compilation-perl--podchecker.txt"
               ((24 "t-compilation-perl.pod" 3)
                (25 "t-compilation-perl.pod" 99)
                (26 "t-compilation-perl.pod" 5)
                (27 "t-compilation-perl.pod" 7)))

              ("t-compilation-perl--Devel::Backtrace.txt"
               ((30 "examples/basic.pl" 12)
                (31 "devel/d-compilation-perl-backtrace.pl" 38)
                (34 "devel/d-compilation-perl-backtrace.pl" 123)
                (38 "devel/d-compilation-perl-backtrace.pl" 38)))))


    (message "  %d tests" count)
    count))
