;; Copyright 2011, 2012, 2019 Kevin Ryde

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()

  ;; Xemacs 21 dired wants the default "ls" time formatting
  ;; (Emacs dired can accept most forms.)
  (setenv "TIME_STYLE" nil)

  (let ((count 0))

    ;;-------------------------------------------------------------------------
    ;; load

    (require 'dired-visit-history)

    ;; no leftovers in emacs21 byte compiling defadvice
    (when (and (boundp 'byte-compile-unresolved-functions)
               byte-compile-unresolved-functions)
      (error "byte-compile-unresolved-functions after load: %S"
             byte-compile-unresolved-functions))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------
    ;; dired-visit-history--add-to-history

    ;; exercise the defun only
    ;; (message "dired-visit-history--add-to-history is %S"
    ;;          (symbol-function 'dired-visit-history--add-to-history))
    (unless (symbolp (symbol-function 'dired-visit-history--add-to-history))
      (let (my-hist)

        (dired-visit-history--add-to-history 'my-hist "foo")
        (unless (equal my-hist '("foo"))
          (error "history not set"))
        (setq count (1+ count))

        (dired-visit-history--add-to-history 'my-hist "bar")
        (unless (equal my-hist '("bar" "foo"))
          (error "history not prepended"))
        (setq count (1+ count))

        (dired-visit-history--add-to-history 'my-hist "bar")
        (unless (equal my-hist '("bar" "foo"))
          (error "history repeat should not be added"))
        (setq count (1+ count))

        (let ((minibuffer-history-uniquify nil))
          (dired-visit-history--add-to-history 'my-hist "foo")
          (unless (equal my-hist '("foo" "bar" "foo"))
            (error "history later repeat should add, got %S" my-hist))
          (setq count (1+ count)))

        (let ((minibuffer-history-uniquify t))
          (dired-visit-history--add-to-history 'my-hist "bar")
          (unless (equal my-hist '("bar" "foo" "foo"))
            (error "minibuffer-history-uniquify deletes, got %S" my-hist))
          (setq count (1+ count)))

        (let ((minibuffer-history-minimum-string-length 2))
          (dired-visit-history--add-to-history 'my-hist "x")
          (unless (equal my-hist '("bar" "foo" "foo"))
            (error "minibuffer-history-minimum-string-length no add"))
          (setq count (1+ count))

          (dired-visit-history--add-to-history 'my-hist "xy")
          (unless (equal my-hist '("xy" "bar" "foo" "foo"))
            (error "minibuffer-history-minimum-string-length yes add"))
          (setq count (1+ count)))

        (let ((history-length 2))
          (dired-visit-history--add-to-history 'my-hist "xyzzy")
          (unless (equal my-hist '("xyzzy" "xy"))
            (error "history-length truncate, got %S" my-hist))
          (setq count (1+ count))

          (setq my-hist nil)
          (dired-visit-history--add-to-history 'my-hist "one")
          (unless (equal my-hist '("one"))
            (error "history-length no truncate nil, got %S" my-hist))
          (setq count (1+ count)))

        (setq my-hist '("a" "b" "c"))
        (put 'my-hist 'history-length 1)
        (dired-visit-history--add-to-history 'my-hist "blah")
        (unless (equal my-hist '("blah"))
          (error "history-length truncate from property, got %S" my-hist))
        (setq count (1+ count))

        (setq my-hist 123)
        (dired-visit-history--add-to-history 'my-hist "foo")
        (unless (equal my-hist 123)
          (error "non-list history should be ignored"))
        (setq count (1+ count))

        ))

    ;;-------------------------------------------------------------------------

    (setq file-name-history nil)

    (dired ".")
    (dired-goto-file (expand-file-name "t-compilation-perl--Glib.txt"))
    (message "%S" (count-lines (point-min) (point)))
    (dired-view-file)
    (kill-buffer (current-buffer))
    (when (member (expand-file-name "t-compilation-perl--Glib.txt")
                  file-name-history)
      (error "Oops, added to history when never enabled\nfile-name-history is %S"
             file-name-history))

    (dired-visit-history-enable)

    (dired ".")
    (dired-goto-file (expand-file-name "t-compilation-perl--Glib.txt"))
    (dired-find-file)
    (kill-buffer (current-buffer))
    (unless (member (expand-file-name "t-compilation-perl--Glib.txt")
                    file-name-history)
      (error "Oops, not added to history when enabled"))

    (dired-visit-history-disable)

    (dired ".")
    (dired-goto-file (expand-file-name "t-compilation-perl--podchecker.txt"))
    (dired-find-file)
    (kill-buffer (current-buffer))
    (when (member (expand-file-name "t-compilation-perl--podchecker.txt")
                  file-name-history)
      (error "Oops, added to history when disabled"))


    ;;-------------------------------------------------------------------------
    ;; unload-feature

    (dired-visit-history-enable)

    (add-to-list 'load-path (expand-file-name "."))
    (require 'my-unload)

    (if (not my-unload-feature-function-p)
        (message "skip, no unload-feature function")

      (let ((feature 'dired-visit-history))
        (unless (my-advice-by-name feature)
          (error "oops, didn't find any advice"))

        (unload-feature feature t)

        (let ((lst (my-advice-by-name feature)))
          (when lst
            (error "unload-feature leftover advice: %S" lst)))
        (setq count (1+ count))))

    ;;-------------------------------------------------------------------------
    ;; unload-feature -- with advice unloaded first

    (let ((feature 'dired-visit-history))
      (if (featurep feature)
          (unload-feature feature t))
      (require feature)
      ;; if 'advice can be unloaded (no dependency expressed on it), then
      ;; see FEATURE unloads successfully too
      (when (condition-case nil
                (progn (unload-feature 'advice) t)
              (error nil))
        (unload-feature feature t)))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------
    ;; dired-find-file,dired-view-file -- after unload-feature

    (if (featurep 'dired-visit-history)
        (unload-feature 'dired-visit-history))
    (dired ".")
    (dired-goto-file (expand-file-name "t-compilation-ledger.txt"))

    (dired-find-file)
    (kill-buffer (current-buffer))
    (setq count (1+ count))

    (dired-view-file)
    (kill-buffer (current-buffer))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------
    (bury-buffer)
    (message "  %d tests" count)
    count))
