;; Copyright 2007, 2009, 2010 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)

  (let ((count 0)
        (tempbuf "*t-xml-coding--temp*"))
    (switch-to-buffer tempbuf)

    ;;-------------------------------------------------------------------------
    ;; load

    (require 'xml-coding)

    ;; no leftovers in emacs21 byte compiling defadvice
    (when (and (boundp 'byte-compile-unresolved-functions)
               byte-compile-unresolved-functions)
      (error "byte-compile-unresolved-functions after load: %S"
             byte-compile-unresolved-functions))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------

    (cond
     ((fboundp 'sgml-xml-auto-coding-function)
      (message "  skip, feature already builtin to this emacs"))

     ((not (or (boundp 'auto-coding-functions)
               (fboundp 'set-auto-coding)))
      (message "  skip, not a known coding setup style"))

     (t

      ;;-----------------------------------------------------------------------
      ;; xml-auto-coding-function
      (setq-default case-fold-search nil)
      (require 'xml-coding)

      (mapcar (lambda (elem)
                (let ((want (car elem))
                      (str  (cadr elem)))
                  (if (and (eq want 'utf-8)
                           (not (memq 'utf-8 (coding-system-list))))
                      (message "  skip utf-8 not available")

                    (erase-buffer)
                    (insert str)
                    (goto-char (point-min))
                    (let ((got (xml-auto-coding-function (- (point-max)
                                                            (point-min)))))
                      (unless (equal got want)
                        (error "got %s want %s" got want)))
                    (setq count (1+ count)))))

              '((iso-8859-1 "<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>")
                (utf-8 "<?xml encoding='utf-8'?>")
                (utf-8 "<?xml encoding=\"utf-8\"?>")
                (utf-8 "\n\n<?xml version=\"1.0\" encoding=\"utf-8\"?>")))

      ;;-----------------------------------------------------------------------
      ;; unload-feature

      (add-to-list 'load-path (expand-file-name "."))
      (require 'my-unload)

      (let ((feature 'xml-coding))
        (unless (my-advice-by-name feature)
          (error "oops, didn't find any advice"))

        (unload-feature feature t)

        (let ((lst (my-advice-by-name feature)))
          (when lst
            (error "unload-feature leftover advice: %S" lst)))
        (setq count (1+ count))

        (when (fboundp 'xml-auto-coding-function)
          (error "`xml-auto-coding-function' not unloaded"))
        (setq count (1+ count)))


      ;;-----------------------------------------------------------------------
      ;; unload-feature -- set-auto-coding still works

      (if (not (fboundp 'set-auto-coding))
          (message "skip, no `set-auto-coding'")
        (require 'xml-coding)
        (unload-feature 'xml-coding t)
        (set-auto-coding "t-xml-coding.xml" 0)
        (setq count (1+ count)))


      ;;-----------------------------------------------------------------------
      ;; unload-feature -- with advice unloaded first

      (let ((feature 'xml-coding))
        (if (featurep feature)
            (unload-feature feature t))
        (require feature)
        ;; if 'advice can be unloaded (no dependency expressed on it), then
        ;; see FEATURE unloads successfully too
        (when (condition-case nil
                  (progn (unload-feature 'advice) t)
                (error nil))
          (unload-feature feature t)))
      (setq count (1+ count))))


    ;;-----------------------------------------------------------------------
    (bury-buffer)
    (message "  %d tests" count)
    count))
