;; Copyright 2009, 2010, 2011, 2016 Kevin Ryde

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.



;;-----------------------------------------------------------------------------
;; cvs with concat

(compile "emacs-cvs-b -l ~/lisp/byte-opt-concat.el -Q -batch -f batch-byte-compile /down/emacs/lisp/*.el")


;;-----------------------------------------------------------------------------
;; eg.

;; concat - xml-parse-dtd regexps
;; insert - bookmark-insert-file-format-version-stamp
;; princ-list - checkdoc-interactive-loop

;;-----------------------------------------------------------------------------

(progn
  (require 'byte-opt-concat)
  (defun foo (x)
    (concat x "ab" "cd" "ef" x))
  (defun foo (x)
    (concat "ab" "cd"))
  (defun foo (x)
    (insert-before-markers x ?a "a" y "b" ?c "d" x "e" "f"))
  (defun foo (x)
    (insert ?\n))
  (defun foo (x)
    (insert ?/ ?>))
  (defun foo (x)
    (1- most-positive-fixnum))
  (defun foo (x)
    (- 1 2 3 x 4 5))
  (byte-compile 'foo)
  (disassemble 'foo))

(byte-optimize-insert '(insert "\n"))
(byte-optimize-insert '(insert ?\n))
(byte-optimize-insert '(insert "a" ""))
(byte-optimize-concat '(concat "a" ""))

;;-----------------------------------------------------------------------------
;; +

(get '+ 'byte-optimizer)
(byte-optimize-pairwise '(+ 1 2 x) 'numberp '+)

(progn
  ;; (load "/home/gg/lisp/byte-opt-concat.el")
  ;; (require 'byte-opt-concat)
  (defun foo (x)
    (+ 1 2 x 3 4 5))
  (byte-compile 'foo)
  (disassemble 'foo))


;;-----------------------------------------------------------------------------
;; -

(get '- 'byte-optimizer)
(byte-optimize-pairwise '(- 5 1 x 5 1) 'numberp '- '+)

(progn
  (load "/home/gg/lisp/byte-opt-concat.el")
  (defun foo (x)
    (- 5 1 x 5 1))
  (byte-compile 'foo)
  (disassemble 'foo))


;;-----------------------------------------------------------------------------
;; *

(get '* 'byte-optimizer)
(byte-optimize-pairwise '(* 2 3 x 4 5) 'numberp '*)
(put '* 'byte-optimizer 'byte-optimize-multiply)
(put '* 'byte-optimizer nil)

(progn
  ;; (require 'byte-opt-concat)
  (defun foo (x)
    (* 2 3 x 4 5))
  (byte-compile 'foo)
  (disassemble 'foo))

(put '* 'byte-optimizer 'byte-optimize-*)

(defadvice byte-optimize-multiply (before byte-opt-concat activate)
  ;; (* 1 2 (foo)) -> (* 3 (foo))
  (setq form (byte-optimize-pairwise form 'numberp '*)))


;;-----------------------------------------------------------------------------
;; princ-list

(grep "(find /down/emacs/lisp -name \\*.el -print0 ; find /usr/share/emacs/site-lisp -name \\*.el -print0) | xargs -0 grep -nH 'princ-list'")

(grep "(find /down/emacs/lisp -name \\*.el -print0 ; find /usr/share/emacs/site-lisp -name \\*.el -print0) | xargs -0 fgrep -nH '(princ-list \"'")

(require 'byte-opt)
(require 'byte-opt-concat)
(progn
  (defun foo (x)
    (dolist (x '(1 2 3))
      (bar x)))
  (defun foo (x)
    (mapc 'princ '("x")))
  (defun foo (x)
    (princ-list "x" "y" "z"))
  (byte-compile 'foo)
  (disassemble 'foo))
(get 'princ-list 'byte-optimizer)

(let ((standard-output (current-buffer)))
  (foo 1)
  '(princ-list "x" "y" "z"))

(progn
  (put 'princ-list 'byte-optimizer nil)
  (defsubst princ-list (&rest args)
    (mapc 'princ args)
    (princ "\n"))
  (put 'princ-list 'byte-optimizer 'byte-optimize-princ-list))

(progn
  (defun foo (x)
    (princ-list "x" "y" "z"))
  (byte-compile 'foo)
  (disassemble 'foo))

;;-----------------------------------------------------------------------------
;; mapc

(grep "(find /down/emacs/lisp -name \\*.el -print0 ; find /usr/share/emacs/site-lisp -name \\*.el -print0) | xargs -0 grep -nH \"[(]mapc .* '[(]\"")


;;-----------------------------------------------------------------------------
;; subr-arity

(let (lst)
  (require 'cl)
  (mapatoms (lambda (symbol)
              (when (fboundp symbol)
                (let ((subr (symbol-function symbol)))
                  (when (and (subrp subr)
                             (eq 'many (cdr (subr-arity subr))))
                    (push symbol lst))))))
  (dolist (symbol (sort* lst 'string< :key 'symbol-name))
    (insert (format "%S\n" symbol))))



;;-----------------------------------------------------------------------------

(decode-char 'ucs 8226)
(decode-char 'iso-8859-1 176)
(aref (decode-coding-string (string 176) 'iso-8859-1) 0)

    (when changed
      (require 'cl)
      (let ((x (cdr ret)))
        (remove-if 'stringp ret)
        (remove-if (lambda (arg) (eq (car-safe arg) 'regexp-quote)) ret)
        (when x
          (if (member t (mapcar 'characterp form))
              (message "adjacent strings and characters: %d %S"
                       (- (length form) (length ret))
                       form)
            (message "adjacent strings: %d %S"
                     (- (length form) (length ret))
                     form)))))

;; `concat' is `pure' too, so if
;; (defun byte-optimize-concat (form)
;;   (setq form (byte-optimize-adjacent-strings form))
;;   '(if (and (= 2 (length form))
;;             (stringp (cadr form)))
;;        (cadr form)  ;; (concat "foo") -> "foo"
;;      form)
;;   form)

;; ;; Adjacent constant strings (etc) concatted.  `concat' is `pure', so
;; ;; a call with every arg constant is optimized already, but this code
;; ;; recognises a mixture of constants and non-constants.
;; 
;; (defun byte-optimize-adjacent (form pred)
;;   (let ((arglist   form)
;; 	(ret       (list (car form)))
;; 	(optimized nil))
;;     (while arglist
;;       (let ((arg (car arglist)))
;;         (setq arglist (cdr arglist))
;; 	(if (funcall pred arg)
;;             (let ((run (list arg)))
;;               (while (and arglist
;;                           (funcall pred (car arglist))) ;; more passing PRED
;;                 (push (car arglist) run)
;;                 (setq arglist (cdr arglist)))
;;               (setq arg (apply (car form)
;;                                (mapcar 'eval (nreverse run))))
;;               (if (or (cdr run)
;;                       (not (equal arg (car run))))
;;                   (setq optimized t))))
;; 	(push arg ret)))
;; 
;;     (setq ret (nreverse ret))
;; 
;;     (when (and optimized
;;                (= 2 (length ret)))
;;       (setq ret (cadr ret)))
;; 
;;     (if (and optimized (listp ret))
;; 	(message "adjacent mixture:\n%S\n%S\n" form ret))
;; 
;;     (if optimized
;;         ret
;;       form)))
;; 
;; (defun byte-optimize-concat (form)
;;   (byte-optimize-adjacent
;;    form (lambda (obj)
;;           (or (stringp obj)
;;               (and (or (and (consp obj)
;;                             (eq 'quote (car obj))
;;                             (progn
;;                               (setq obj (cadr obj))
;;                               t))
;;                        (vectorp obj))
;;                    (not (memq nil (mapcar 'characterp obj))))))))
;; 
;; (defun byte-optimize-insert (form)
;;   (byte-optimize-adjacent form 'stringp))
;; 
;; 
;; (let* ((s   (string-make-unibyte "\377"))
;;        (m   (string-make-multibyte "\377"))
;;        (got (concat s m)))
;;   (multibyte-string-p got))


;; (defconst byte-opt-concat--nonascii-translation-p
;;   (let ((nonascii-translation-table (make-translation-table '((255 . ?X)))))
;;     (eq ?X (unibyte-char-to-multibyte 255)))
;;   "Non-nil if `nonascii-translation-table' is used by unibyte to multibyte.")
;; 
;; (let ((uni   (string-make-unibyte (string 255)))
;;       (multi (string-make-multibyte "x")))
;;   (let ((nonascii-translation-table (make-translation-table '((127 . 1)
;; 							      (255 . 1)))))
;;     (setq foo (concat multi uni))))
;; (aref foo 0)
;; 
;; (defun byte-opt-concat-mixbyteness-p (str1 str2)
;;   "Return non-nil if STR1 and STR2 have compilable byteness.
;; Prior to emacs23 multibyte/unibyteness.
;;   (or (not byte-opt-concat--nonascii-translation-p)
;;       (eq (multibyte-string-p str1)
;;           (multibyte-string-p str2))))
;; 
;; 
;;                       (byte-opt-concat-mixbyteness-p arg (car arglist))
;; 
;; ;; Unibyte strings are not concatenated with multibyte because in Emacs 22
;; ;; and earlier that depended on runtime `nonascii-translation-table'.
;; ;; A concat here would mean the .elc would have to be tagged for 23 and up.
;; ;; Don't force that yet.  Mixtures of multibyte and unibyte literals will be
;; ;; extremely rare anyway, probably only possible out of a macro or an
;; ;; eval-when-compile, so prefer compatibility.
;; ;;









;; (defun byte-optimize-stringp-or-charp (obj)
;;   (or (stringp obj) (characterp obj)))

;; (defun byte-optimize-concatenate-adjacent (form &optional pred)
;;   "Return FORM with adjacent string arguments concatenated.
;; PRED is the type of arguments to concatenate, either `stringp' or
;; `byte-optimize-stringp-or-charp'.
;; If concatenations are done the return is a new form, if not the
;; return is FORM unchanged."
;;   (let ((ret     (list (car form)))
;; 	(arglist (cdr form))
;; 	changed)
;;     (while arglist
;;       (let ((arg (pop arglist)))
;;         (if (funcall pred arg)
;;             (while (and arglist (funcall pred (car arglist)))
;;               (let ((arg2 (pop arglist)))
;;                 (if (characterp arg)  (setq arg  (char-to-string arg)))
;;                 (if (characterp arg2) (setq arg2 (char-to-string arg2)))
;;                 (setq arg (concat arg arg2))
;;                 (setq changed t))))
;; 	(push arg ret)))
;; 
;;     (if (and changed
;;              (if (eq 'concat (car form))
;;                  (>= (length form) 3)
;;                (>= (length form) 2)))
;;         (message "changed from %S" form))
;; 
;;     (if changed
;;         (nreverse ret)
;;       form)))

;; (put 'insert                            'byte-optimizer 'byte-optimize-insert)
;; (put 'insert-and-inherit                'byte-optimizer 'byte-optimize-insert)
;; (put 'insert-before-markers             'byte-optimizer 'byte-optimize-insert)
;; (put 'insert-before-markers-and-inherit 'byte-optimizer 'byte-optimize-insert)
;; (defun byte-optimize-insert (form)
;;   ;; (insert "x" "y" ?z (foo)) -> (insert "xyz" (foo))
;;   (setq form (byte-optimize-concatenate-adjacent
;;               form 'byte-optimize-stringp-or-charp))
;; 
;;   ;; (insert "x") -> (insert ?x) since a char is less memory.
;;   ;; But only for ascii, in case high char values change again they way they
;;   ;; did for unicode in Emacs 23 (strings in a .elc are decoded according to
;;   ;; the file charset, plain numbers are not).
;;   (when (and (= 2 (length form))
;;              (stringp (cadr form))
;;              (= 1 (length (cadr form)))
;;              (string-match "[[:ascii:]]" (cadr form)))
;;     (setq form (list (car form)
;;                      (string-to-char (cadr form)))))
;; 
;;   form)


;;-----------------------------------------------------------------------------
;; add-to-list

(progn
  (defun foo (x)
    (add-to-list 'foo 123))
  (byte-compile 'foo)
  (disassemble 'foo))

(progn
  (defun foo (x)
    (push 123 foo))
  (byte-compile 'foo)
  (disassemble 'foo))
